@@ -7,36 +7,37 @@ import pandas as pd
 from scipy.interpolate import CubicSpline, interp1d
 from statsmodels.nonparametric.smoothers_lowess import lowess
 from typing import List, Callable, Union, Optional
+from .utils import mad
 from ._names import *
 
 
-def input_na(df: pd.DataFrame, classes: pd.Series, mode: str) -> pd.DataFrame:
-    """
-    Fill missing values.
-    
-    Parameters
-    ----------
-    df : pd.DataFrame
-    classes: ps.Series
-    mode : {'zero', 'mean', 'min'}
-    
-    Returns
-    -------
-    filled : pd.DataFrame    
-    """
-    if mode == "zero":
-        return df.fillna(0)
-    elif mode == "mean":
-        return (df.groupby(classes)
-                .apply(lambda x: x.fillna(x.mean()))
-                .droplevel(0))
-    elif mode == "min":
-        return (df.groupby(classes)
-                .apply(lambda x: x.fillna(x.min()))
-                .droplevel(0))
-    else:
-        msg = "mode should be `zero`, `mean` or `min`"
-        raise ValueError(msg)
+# def input_na(df: pd.DataFrame, classes: pd.Series, mode: str) -> pd.DataFrame:
+#     """
+#     Fill missing values.
+#
+#     Parameters
+#     ----------
+#     df : pd.DataFrame
+#     classes: ps.Series
+#     mode : {'zero', 'mean', 'min'}
+#
+#     Returns
+#     -------
+#     filled : pd.DataFrame
+#     """
+#     if mode == "zero":
+#         return df.fillna(0)
+#     elif mode == "mean":
+#         return (df.groupby(classes)
+#                 .apply(lambda x: x.fillna(x.mean()))
+#                 .droplevel(0))
+#     elif mode == "min":
+#         return (df.groupby(classes)
+#                 .apply(lambda x: x.fillna(x.min()))
+#                 .droplevel(0))
+#     else:
+#         msg = "mode should be `zero`, `mean` or `min`"
+#         raise ValueError(msg)
 
 
 def average_replicates(data: pd.DataFrame, sample_id: pd.Series,
@@ -70,9 +71,9 @@ def average_replicates(data: pd.DataFrame, sample_id: pd.Series,
 
 def correct_blanks(df: pd.DataFrame, classes: pd.Series,
                    corrector_classes: List[str], process_classes: List[str],
-                   factor: float == 1,
+                   factor: float = 1.0, robust: bool = True,
                    mode: Union[str, Callable] = "mean",
-                   process_blanks: bool = True) -> pd.DataFrame:
+                   process_blanks: bool = False) -> pd.DataFrame:
     """
     Correct samples using blanks.
 
@@ -98,10 +99,17 @@ def correct_blanks(df: pd.DataFrame, classes: pd.Series,
         Data with applied correction
 
     """
+    if robust:
+        mean_func = lambda x: x.median()
+        std_func = mad
+    else:
+        mean_func = lambda x: x.mean()
+        std_func = lambda x: x.std()
+
     corrector = {"max": lambda x: x.max(),
-                 "mean": lambda x: x.mean(),
-                 "lod": lambda x: x.mean() + 3 * x.std(),
-                 "loq": lambda x: x.mean() + 10 * x.std()}
+                 "mean": lambda x: mean_func,
+                 "lod": lambda x: mean_func(x) + 3 * std_func(x),
+                 "loq": lambda x: mean_func(x) + 10 * std_func(x)}
     if hasattr(mode, "__call__"):
         corrector = mode
     else:
@@ -297,7 +305,8 @@ def check_qc_prevalence(data_matrix: pd.DataFrame,
 
 def loess_interp(ft_data: pd.Series, order: pd.Series, qc_index: pd.Index,
                  sample_index: pd.Index, frac: float, interpolator: Callable,
-                 n_qc: Optional[int] = None) -> pd.Series:
+                 n_qc: Optional[int] = None, method: str = "multiplicative"
+                 ) -> pd.Series:
     """
     Applies LOESS-correction interpolation on a feature. Auxiliary function of
     batch_corrector_func
@@ -313,19 +322,42 @@ def loess_interp(ft_data: pd.Series, order: pd.Series, qc_index: pd.Index,
     interpolator: Callable
     n_qc: int, optional
     Number of QCs involved in mean calculation. If None, all QCs are involved.
-
+    method : {"multiplicative", "additive"}
+        Method used to model variation.
     Returns
     -------
     pd.Series
     """
     if n_qc is None:
         n_qc = qc_index.size
+
     qc_median = ft_data[qc_index[:n_qc]].median()
-    qc_loess = _loocv_loess(order[qc_index],
-                            ft_data[qc_index] - qc_median,
-                            interpolator, frac=frac)
+    # if there are several 0 values the median may be 0, this prevent against
+    # such cases
+    if np.isclose(qc_median, 0.0):
+        qc_median = ft_data[qc_index[:n_qc]].mean()
+
+    qc_loess = _loocv_loess(order[qc_index], ft_data[qc_index], interpolator,
+                            frac=frac)
     interp = interpolator(order[qc_index], qc_loess)
-    ft_data[sample_index] -= interp(order[sample_index])
+
+    if method == "additive":
+        bias = interp(order[sample_index]) - qc_median
+        ft_data[sample_index] -= bias
+    elif method == "multiplicative":
+        qc_smooth = interp(order[sample_index])
+        # smoothed values close to zero can become negative. As we are using
+        # a multiplicative correction, this prevents changing signs while
+        # applying the correction
+        if qc_median <= 0.0:
+            factor = 0 * qc_smooth
+        else:
+            qc_smooth[qc_smooth <= 0] = qc_median
+            factor = qc_median / qc_smooth
+        ft_data[sample_index] *= factor
+    else:
+        msg = "Valid methods are `additive` or `multiplicative`."
+        raise ValueError(msg)
     return ft_data
 
 
@@ -333,7 +365,8 @@ def batch_corrector_func(df_batch: pd.DataFrame, order: pd.Series,
                          classes: pd.Series, frac: float,
                          interpolator: Callable, qc_classes: List[str],
                          sample_classes: List[str],
-                         n_qc: Optional[int] = None) -> pd.DataFrame:
+                         n_qc: Optional[int] = None,
+                         method: str = "multiplicative") -> pd.DataFrame:
     """
     Applies LOESS correction - interpolation on a single batch. Auxiliary
     function of interbatch_correction.
@@ -349,6 +382,8 @@ def batch_corrector_func(df_batch: pd.DataFrame, order: pd.Series,
     sample_classes: list[str]
     n_qc: int, optional
     Number of QCs involved in mean calculation. If None, all QCs are involved.
+    method : {"multiplicative", "additive"}
+        Method used to model variation.
 
     Returns
     -------
@@ -359,8 +394,10 @@ def batch_corrector_func(df_batch: pd.DataFrame, order: pd.Series,
     sample_index = classes.isin(sample_classes)
     sample_index = sample_index[sample_index].index
     df_batch.loc[sample_index, :] = \
-        (df_batch.apply(loess_interp, args=(order, qc_index, sample_index,
-                                            frac, interpolator), n_qc=n_qc))
+        (df_batch.apply(loess_interp,
+                        args=(order, qc_index, sample_index, frac,
+                              interpolator),
+                        n_qc=n_qc, method=method))
     return df_batch
 
 
@@ -370,7 +407,8 @@ def interbatch_correction(df: pd.DataFrame, order: pd.Series, batch: pd.Series,
                           frac: Optional[float] = None,
                           interpolator: Optional[str] = "splines",
                           n_qc: Optional[int] = None,
-                          process_qc: bool = True
+                          process_qc: bool = True,
+                          method: str = "multiplicative"
                           ) -> pd.DataFrame:
     r"""
     Correct instrument response drift using LOESS regression [1, 2]
@@ -399,8 +437,8 @@ def interbatch_correction(df: pd.DataFrame, order: pd.Series, batch: pd.Series,
         involved.
     process_qc : bool
         If True, applies correction to QC samples.
-
-
+    method : {"multiplicative", "additive"}
+        Method used to model variation.
 
     Returns
     -------
@@ -449,24 +487,32 @@ def interbatch_correction(df: pd.DataFrame, order: pd.Series, batch: pd.Series,
         return batch_corrector_func(df_group, order[df_group.index],
                                     classes[df_group.index], frac,
                                     interp_func, corrector_classes,
-                                    process_classes, n_qc=n_qc)
+                                    process_classes, n_qc=n_qc,
+                                    method=method)
 
     # intra batch correction
     corrected = df.groupby(batch).apply(corrector_helper)
 
     # inter batch mean alignment
-
     def batch_mean_func(df_group):
         batch_mean = (df_group[classes[df_group.index].isin(corrector_classes)]
                       .mean())
-        df_group[classes[df_group.index].isin(process_classes)] = \
-            df_group[classes[df_group.index].isin(process_classes)] - batch_mean
+        is_process_sample = classes[df_group.index].isin(process_classes)
+        if method == "additive":
+            df_group[is_process_sample] -= batch_mean
+        else:
+            batch_mean[batch_mean <= 0] = np.nan
+            df_group[is_process_sample] /= batch_mean
+            df_group.fillna(0)
         return df_group
 
     global_median = corrected[classes.isin(corrector_classes)].median()
     corrected = corrected.groupby(batch).apply(batch_mean_func)
-    corrected.loc[classes.isin(process_classes), :] = \
-        corrected.loc[classes.isin(process_classes), :] + global_median
+    process_mask = classes.isin(process_classes)
+    if method == "additive":
+        corrected.loc[process_mask, :] += global_median
+    else:
+        corrected.loc[process_mask, :] *= global_median
     corrected[corrected < 0] = 0
 
     return corrected
